<?
/*
(C) 2004-2013 DatuX - info@datux.nl

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

include_once("../common.php");
require_once("../file.php");

$BCKCONF="/etc/backup.conf";

$bcksettings=FileReadSettings($BCKCONF);

function BackupWriteSettings()
{
	global $BCKCONF;
	global $bcksettings;
	FileWriteSettings($BCKCONF,$bcksettings);
	FileChmod($BCKCONF, "600");
	LogInfo(_BackupSettingsSaved());
}

function TestTape()
{
	global $bcksettings;
	return ( Cmd( "mt -f ".$bcksettings[TAPEDEV]." status", '', $out, $err ) == 0 );
}

function TestSmb($new_smbserver,$new_smbshare,$new_smbdomain,$new_smbuser,$new_smbpasswd)
{
	$ret=Cmd( "smbclient -c 'put /etc/HOSTNAME syn3test.txt ; rm syn3test.txt' -U '$new_smbuser' -W '$new_smbdomain' '//$new_smbserver/$new_smbshare' '$new_smbpasswd'", '', $out, $err );

	if ($ret!=0 || $out)
	{
		return implode(", ",$out);
	}
	return false;
}

function TestRsync($new_rsyncserver,$new_rsyncpasswd,$new_rsyncdir,$new_rsyncport)
{
	#try to create a directory and use all the rsync options we need, to make sure everything works
	mkdir("/tmp/syn3backuptest");
	$ret=Cmd("PASSWD='$new_rsyncpasswd' /usr/backup/sshpass rsync --rsh 'ssh -p $new_rsyncport -o NumberOfPasswordPrompts=1 -o StrictHostKeyChecking=no -o PubkeyAuthentication=no' -r --rsync-path='rsync --fake-super' /tmp/syn3backuptest/ '$new_rsyncserver:$new_rsyncdir/syn3backupmeta'",
			 '', $out, $err);
	rmdir("/tmp/syn3backuptest");
	if ($ret!=0)
	{
		return implode(" ",$out);
	}
	return false;
}


//gets the status of the backup/restore
function GetStatus()
{
	global $bcksettings;
	$raw=FileRead($bcksettings[LOG]);
	if (!Cmd("pgrep -xo 'restore|backup'", '', $out, $err))
	{
		$running=1;
	}
	else
	{
		$running=0;
	}

	$logs=BackupGetLog();

	//gets the last perc and prog status
	$perc=0;
	$timeleft='';
	$logerrors=0;
	foreach ($logs as $log)
	{
		if ($log[type]=="STAT")
		{
			$stat=$log[txt];
		}
		elseif($log[type]=="PERC")
		{
			list($perc,$timeleft)=explode(":",$log[txt]);
		}
		else
		{
			$lastmsg=$log[txt];
			$lasttype=$log[type];
		}
		if ($log[type]=="ERROR")
		{
			$logerrors++;
		}
	}
	return array($running,$stat,$perc,$timeleft,$lasttype,$lastmsg,$logerrors);
}

//read backuplogs
//returns:
//$ret[0][type]= INFO or STAT or ERROR or PERC
//$ret[0][txt]='log text'
function BackupGetLog()
{
	global $bcksettings;
	$raw=FileRead($bcksettings[LOG]);
	foreach ($raw as $line)
	{
		if (preg_match('/^(INFO|STAT|PERC):/',$line))
		{
			list($msgtype,$msgcontent)=explode(":",$line,2);
		}
		else
		{
			//the rest of the output are errors
			$msgtype="ERROR";
			$msgcontent=$line;
		}
		$ret[]=array('type'=>$msgtype, 'txt'=>$msgcontent);
	}
	return $ret;
}

function Abort()
{
	Cmd("killall -g restore backup restorefiles", '', $out, $err);
	sleep(1);
}

function StartBackup()
{
	global $bcksettings;

	list($running)=GetStatus();
	if (!$running)
	{
		$backupscript=$bcksettings['SCRIPTDIR']."/backup";
		BgExec("$backupscript now");
		sleep(1);
	}
}

function StartRestore($backupname)
{
	global $bcksettings;

	list($running)=GetStatus();
	if (!$running)
	{
		$restorescript=$bcksettings['SCRIPTDIR']."/restore";
		BgExec("$restorescript '$backupname'");
		sleep(1);
	}
}

//makes a syn3backup file type human readable
function File2Text($filename)
{
	//remove extra metadata path stuff, if any
	$filename=preg_replace("@.*syn3backupmeta/@", "",$filename);

	list($bla,$flags,$stamp,$host)=explode("_",$filename);
	$weekday=date(	"w"	,$stamp);
	$month=date(	"n"	,$stamp);
	$week=date(		"W"	,$stamp);

	$backuptype="";
	if (strchr($flags,'M'))
		$txt[]=_MonthlyBackup(MonthName($month));

	if (strchr($flags,'W'))
		$txt[]=_WeeklyBackup($week);

	if (strchr($flags,'D'))
		$txt[]=_DailyBackup(DayName($weekday));

	$backuptype=implode("<br>",$txt);
	return(array(Time2Text($stamp),$backuptype,$host));
}

#returns a list of available backups
#$ret[backupfilename]=timestamp
function GetList()
{
	global $bcksettings;
	$files=SafeExec($bcksettings['SCRIPTDIR']."/list");
	foreach($files as $file)
	{
		list($bla,$bla,$timestamp)=explode("_",basename($file));
		$ret[$file]=(int)$timestamp;
	}
	arsort($ret, SORT_NUMERIC);
	return $ret;
}

function GetMediumName()
{
	global $bcksettings;
	if ($bcksettings[mode]=="tape")
		return ("tapedrive");

	if ($bcksettings[mode]=="smb")
		return ($bcksettings[smbserver]);

	if ($bcksettings[mode]=="rsync")
		return ($bcksettings[rsyncserver]);
}

# This function returns a list of files/directories from the cachefile.
# It will only return files that are directly below path and its NON recursive.
# (just like the user sees it in the startrestorefiles.php)
# Directories are ending with a slash.
function GetFileList($backupname,$frompath)
{
	global $bcksettings;

	#get the listing of the tape:
	#-grep files that start with the path we want
	#-strip the rest of the path
	#-only return uniq names
	#This will work with HUGE lists as well.
	$frompath=RegexEscape($frompath);
	$frompath=BashQuote($frompath);

	$filelist=SafeExec(
		$bcksettings['SCRIPTDIR']."/listfiles '$backupname' |
			egrep \"^$frompath\" |       # only get specified path
			sed -r \"s.^$frompath..\" |  # strip start of path
			sed 's@/.*$@/@' |            # strip end of path
			sort |                       # only return uniq results
			uniq
	");

	return ($filelist);
}

function RestoreFiles($filename, $filenames2Restore)
{

	global $bcksettings;
	$filenamesStr="";
	$logStr="";
	foreach($filenames2Restore as $filename2Restore) {
		//xfsrestore works with relative paths
		$filename2Restore=ltrim($filename2Restore,"/");
		$filenamesStr=$filenamesStr." \"".BashQuote($filename2Restore)."\"";
		$logStr.="$filename2Restore ";
	}
	SafeExec($bcksettings['SCRIPTDIR']."/restorefiles $filename $filenamesStr");
	LogInfo(_FilesFromBackupRestored($filename,$logStr));
}

?>
